/*
 * copyright (2012) Benoit Gschwind
 * copyright (2012) ARMINES/Ecoles Des Mines de Paris
 */

#ifndef JULIAN_DATE_TIME_H_
#define JULIAN_DATE_TIME_H_

#include <math.h>
#include <float.h>

#ifdef  __cplusplus
extern "C"
{
#endif


typedef struct {
	int year;
	int month;
	int day_of_month;
	int hour;
	int minute;
	double second;
} gregorian_date_time_t;



/* TODO: use sg2 or ??? */
inline static double julian_date_time(int year, int month, int day, int hour,
		int minute, double second) {
	const int IGREG = (15 + 31 * (10 + 12 * 1582)); // Gregorian calendar adopted 15 Oct. 1582
	double jul = 0.0;
	int ja = 0;
	int jy = year;
	int jm = 0;
	if (jy < 0)
		jy = jy + 1;
	if (month > 2) {
		jm = month + 1;
	} else {
		jy = jy - 1;
		jm = month + 13;
	}
	jul = floor(floor(365.25 * jy) + floor(30.6001 * jm) + day + 1720995);
	if (day + 31 * (month + 12 * year) >= IGREG) { // test whether to change to gregorian cal.
		ja = (int) floor(0.01 * jy);
		jul = floor(jul + 2 - ja + floor(0.25 * ja));
	}
	return jul + ((double)hour - 12.0) / 24.0 + (double)minute / (60.0 * 24.0)
			+ (double)second / (60.0 * 60.0 * 24.0);
}

/* inspired by numerical recipes */
inline static void julian_to_jmd(double julian, int * year, int * month, int * day, int * hour, int * min, double * sec) {
	int ijulian = (int)floor(julian + 0.5);
	double rest = julian + 0.5 - floor(julian + 0.5);
	int ja, jalpha, jb, jc, jd, je;
	const int IGREG = (15 + 31 * (10 + 12 * 1582)); // Gregorian calendar adopted 15 Oct. 1582
	if (ijulian >= IGREG) {
		jalpha = (int) ((ijulian - 1867216 - 0.25) / 36524.25);
		ja = ijulian + 1 + jalpha - (int) (0.25 * jalpha);
	} else {
		ja = ijulian;
	}

	jb = ja + 1524;
	jc = (int) (6680.0 + (((jb - 2439870) - 122.1) / 365.25));
	jd = (int)floor(365 * jc + (0.25 * jc));
	je = (int)floor((jb - jd) / 30.6001);
	*day = jb - jd - (int) (30.6001 * je);
	*month = je - 1;
	if (*month > 12)
		*month -= 12;
	*year = jc - 4715;
	if (*month > 2)
		--(*year);
	if (*year <= 0)
		--(*year);

	*hour = (int)floor(rest*24.0);
	*min = (int)floor(rest*24.0*60.0 - floor(rest*24.0)*60.0);
	*sec = (rest*24.0*60.0*60.0 - floor(rest*24.0*60.0)*60.0);
}

inline static void julian_to_jmd2(double julian, gregorian_date_time_t * date) {
	julian_to_jmd(julian, &(date->year), &(date->month), &(date->day_of_month), &(date->hour), &(date->minute), &(date->second));
}

#ifdef  __cplusplus
}
#endif

#endif /* JULIAN_DATE_TIME_H_ */
